#include <math.h>

#include "YRTEngine.h"
#include "YRTUtils.h"

#pragma hdrstop

TDateTime TYRTEngine::GetRenderTime(void)
{
    return FinTime - BegTime;
}

void TYRTEngine::RenderScene(TYRTScene *AScene, TCanvas *ACanvas, TYRTEngineUpdateFunc AUpdateFunc)
{
    BegTime = TDateTime::CurrentTime();

    TYRTRay ray;

    // Set the "eye" to be in the mid-point of the screen,
    // but at a distance equal to the width of the screen:
    ray.Start.x = (ACanvas->ClipRect.Left + ACanvas->ClipRect.Right) / 2;
    ray.Start.y = (ACanvas->ClipRect.Top + ACanvas->ClipRect.Bottom) / 2;
    ray.Start.z = -(ACanvas->ClipRect.Right - ACanvas->ClipRect.Left);

    // ray.Dir will point to consecutive pixels of the screen;
    // the .z component is always equal to the distance of the "eye" from the screen.
    ray.Dir.z = -ray.Start.z;

    for (int t = 0; t < AScene->GetCount(); t++)
    {
        TYRTShape *shape = AScene->GetShape(t);
        shape->RunPrecalculations();
    }

    for (int x = ACanvas->ClipRect.Left; x < ACanvas->ClipRect.Right; x++)
    {
        for (int y = ACanvas->ClipRect.Top; y < ACanvas->ClipRect.Bottom; y++)
        {
            // Set the .x and .y components of the ray from the "eye" to the pixel (x, y):
            ray.Dir.x = -ray.Start.x + x;
            ray.Dir.y = +ray.Start.y - y;

            // Get the color of the closes object in the direction of our ray:
            int colorMin = clBlack;
            float distMin = MAX_FLOAT;

            for (int t = 0; t < AScene->GetCount(); t++)
            {
                TYRTShape *shape = AScene->GetShape(t);
                TYRTVector point;
                TColor color;

                if (shape->GetIntersection(ray, &point, &color))
                {
                    float dist = ray.Start.GetDistanceSqr(point);
                    if (dist < distMin)
                    {
                        distMin = dist;
                        colorMin = color;
                    }
                }
            }

            // Draw the pixel in the color of the closes object:
            ACanvas->Pixels[x][y] = (TColor)colorMin;
        }

        // Update image:
        if (x % 16 == 0 && AUpdateFunc != NULL)
        {
            FinTime = TDateTime::CurrentTime();
            AUpdateFunc(x);
        }
    }

    FinTime = TDateTime::CurrentTime();
}

